import { Fun } from '../utils/index.js';
export declare const FULLPATH: unique symbol;
export declare const EXPORTS: unique symbol;
export declare enum CaseStyle {
    camel = "camel",
    lower = "lower",
    upper = "upper"
}
export type CaseStyleFunction = (filepath: string) => string[];
export type FileLoaderInitializer = (exports: unknown, options: {
    path: string;
    pathName: string;
}) => unknown;
export type FileLoaderFilter = (exports: unknown) => boolean;
export interface FileLoaderOptions {
    /** directories to be loaded */
    directory: string | string[];
    /** attach the target object from loaded files */
    target: Record<string, any>;
    /** match the files when load, support glob, default to all js files */
    match?: string | string[];
    /** ignore the files when load, support glob */
    ignore?: string | string[];
    /** custom file exports, receive two parameters, first is the inject object(if not js file, will be content buffer), second is an `options` object that contain `path` */
    initializer?: FileLoaderInitializer;
    /** determine whether invoke when exports is function */
    call?: boolean;
    /** determine whether override the property when get the same name */
    override?: boolean;
    /** an object that be the argument when invoke the function */
    inject?: Record<string, any>;
    /** a function that filter the exports which can be loaded */
    filter?: FileLoaderFilter;
    /** set property's case when converting a filepath to property list. */
    caseStyle?: CaseStyle | CaseStyleFunction;
    lowercaseFirst?: boolean;
}
export interface FileLoaderParseItem {
    fullpath: string;
    properties: string[];
    exports: object | Fun;
}
/**
 * Load files from directory to target object.
 * @since 1.0.0
 */
export declare class FileLoader {
    static get FULLPATH(): symbol;
    static get EXPORTS(): symbol;
    readonly options: FileLoaderOptions & Required<Pick<FileLoaderOptions, 'caseStyle'>>;
    /**
     * @class
     * @param {Object} options - options
     * @param {String|Array} options.directory - directories to be loaded
     * @param {Object} options.target - attach the target object from loaded files
     * @param {String} options.match - match the files when load, support glob, default to all js files
     * @param {String} options.ignore - ignore the files when load, support glob
     * @param {Function} options.initializer - custom file exports, receive two parameters, first is the inject object(if not js file, will be content buffer), second is an `options` object that contain `path`
     * @param {Boolean} options.call - determine whether invoke when exports is function
     * @param {Boolean} options.override - determine whether override the property when get the same name
     * @param {Object} options.inject - an object that be the argument when invoke the function
     * @param {Function} options.filter - a function that filter the exports which can be loaded
     * @param {String|Function} options.caseStyle - set property's case when converting a filepath to property list.
     */
    constructor(options: FileLoaderOptions);
    /**
     * attach items to target object. Mapping the directory to properties.
     * `app/controller/group/repository.js` => `target.group.repository`
     * @return {Object} target
     * @since 1.0.0
     */
    load(): Promise<object>;
    /**
     * Parse files from given directories, then return an items list, each item contains properties and exports.
     *
     * For example, parse `app/controller/group/repository.js`
     *
     * ```
     * module.exports = app => {
     *   return class RepositoryController extends app.Controller {};
     * }
     * ```
     *
     * It returns a item
     *
     * ```
     * {
     *   properties: [ 'group', 'repository' ],
     *   exports: app => { ... },
     * }
     * ```
     *
     * `Properties` is an array that contains the directory of a filepath.
     *
     * `Exports` depends on type, if exports is a function, it will be called. if initializer is specified, it will be called with exports for customizing.
     * @return {Array} items
     * @since 1.0.0
     */
    protected parse(): Promise<FileLoaderParseItem[]>;
}
